/************************************************************************/
/*									*/
/* 	AB-Videogen.cmd: Creation of the WSE2013-Animation 		*/
/*		"Assumburg with eCS-Flags"				*/
/*									*/
/* Author: Wolfgang Reinken, Germany					*/
/*									*/
/* Date of creation: June  2nd 2013					*/
/* Last update:	     July 12th 2013					*/
/*									*/
/* Source files: 							*/
/* - Assumburg.jpg (own photo 1600x1200 pixels)				*/
/* - WSE11-ECSF_5001.jpg ... WSE11-ECSF_5048.jpg			*/
/*     	This is a sequence of photos extracted from an own video,	*/
/*	showing the eCS-flag during WSE2011 in Haarlem. Extracting 	*/
/*	these photos was done by means of the following programmes:	*/
/*	- ffmpeg: converting the video into a multipicture-jpeg		*/
/*	- mjpeg2jpeg.cmd: Own script to split the mjpeg to single pic's	*/
/*	- PMView: Changing the angle and stretching			*/
/*	- OpenOffice Draw: Eliminating the background by creating masks	*/
/*	- ImpOS2: mixing the flag-pictures with OO created masks	*/
/* - Tuer-offen-4k.jpg (Selection from an own photo of reception area)	*/
/* - Tuer-zu-l-01.jpg ... Tuer-zu-l-42.jpg and				*/
/*   Tuer-zu-r-01.jpg ... Tuer-zu-r-42.jpg				*/
/*	Selection of the left and right door leaf from Assumburg.jpg, 	*/
/*	compressed horizontally with PMView (the number in the file 	*/
/*	name represents the width of the picture)			*/
/* - gmsa-pyp.mp3 (sound for the video, extraction of an internetradio	*/
/*	recording)							*/
/*									*/
/* Work files:								*/
/* - AB-5001.bmp ... AB-5048.bmp					*/
/*	Result of the programme's first cycle created by PMView macros: */
/*	The pictures WSE11-ECSF_50xx.jpg are pasted 4 times to the 4	*/
/*	flagpoles in the following manner:				*/
/*	- AB-5001.bmp: 							*/
/*		flagpole 1 (left):  WSE11-ECSF_5001.jpg			*/
/*		flagpole 2:	    WSE11-ECSF_5013.jpg			*/
/*		flagpole 3: 	    WSE11-ECSF_5025.jpg			*/
/*		flagpole 4 (right): WSE11-ECSF_5037.jpg			*/
/*	- AB-5002.bmp: 							*/
/*		flagpole 1 (left):  WSE11-ECSF_5002.jpg			*/
/*		flagpole 2:	    WSE11-ECSF_5014.jpg			*/
/*		flagpole 3: 	    WSE11-ECSF_5026.jpg			*/
/*		flagpole 4 (right): WSE11-ECSF_5038.jpg			*/
/*			:						*/
/*	- AB-5048.bmp: 							*/
/*		flagpole 1 (left):  WSE11-ECSF_5048.jpg			*/
/*		flagpole 2:	    WSE11-ECSF_5012.jpg			*/
/*		flagpole 3: 	    WSE11-ECSF_5024.jpg			*/
/*		flagpole 4 (right): WSE11-ECSF_5036.jpg			*/
/*	These workfiles form one sequence of waving flags and they are  */
/*	the input of the programme's second cycle			*/
/* - AB-0000.jpg ... AB-0824.jpg					*/
/*	Result of the programme's second cycle, are used by ffmpeg to	*/
/*	create a video							*/
/*									*/
/* Output file:								*/
/* - AB-flag2.mp4 (Video with sound)					*/
/*									*/
/************************************************************************/


/* -------------------------------------------------------------------- */
/*									*/
/* Loading additional DLLs						*/
/*									*/
/* -------------------------------------------------------------------- */

'@start d:\osutil\impos21\impos.exe'
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs
call SysSleep 1
rc = RxFuncAdd("ImgInitiate", "IMPREXX", "ImgInitiate")
rc = ImgInitiate()
call RxFuncAdd 'MathLoadFuncs', 'REXXMATH', 'MathLoadFuncs'
call MathLoadFuncs
"@echo off"


/* -------------------------------------------------------------------- */
/*									*/
/* First program cycle							*/
/*									*/
/* It creates 48 pictures of the Assumburg with 4 eCS flags		*/
/* The sequence of these 48 pictures forms one cyle of waving flags	*/
/* in front of the Assumburg						*/
/*									*/
/* This program cycle takes a while (about one hour). That's why the	*/
/* pictures AB-5001.bmp ... AB-5048.bmp are only created if they do not */
/* exist. The advantage of this behaviour is the posibility to modify 	*/
/* only the second phase.						*/
/* -------------------------------------------------------------------- */

do i=1 to 48

  ic=5000+i
  icm=ic
  adat="AB-"icm".bmp"
  "DIR "adat" 1>nul 2>nul"
  if RC > 0 then
  do
    say "Bild: "i
    ima = ImgLoadImage("Assumburg.jpg")		/* loading background picture "assumburg" 			*/
    imf = ImgLoadImage("WSE11-ECSF_"ic".jpg")	/* loading first prepared flag-picture (with black background) 	*/
    rc  = ImgSetTool(imf, "SELMAGIC")		/* preselection "Macic Selection Tool" of ImpOS2		*/
    say "Click into black background next to the flag, press <ALT>I, <ALT>H and ..."
    "PAUSE" 					/* at this point the following operator actions are necesarry:	*/
	/* 				   - click with "Magic Selection Tool" on the black Background of the open */
 	/*				     picture WSE11-ECSF_50xx.jpg					*/
	/*				   - invert the selection (<ALT>I					*/
	/*				   - switch to Command Line of this program end press any key to continue */
	/*				   - answer the following question (sve changed file) with NO		*/
    rc  = ImgCopyImage(imf)		     /* copy the magic selection to clipboard				*/
    rc  = ImgRevertImage(imf,TRUE)	     /* copy to clipboard						*/
    rc  = ImgCloseImage(imf)		     /* close flag-picture to free memory				*/	
    rc  = ImgPasteImage(ima,0,249,465,TRUE)  /* paste the magic selection (flag without background) to the first flagpole */

    ic=(i+11)//48+1			     /* calculating the number of the second flag picture with a distance of 12 */
					     /* pictures to the first 							*/
    ic=5000+ic				     /* the modulo (//) operation ensures the wrap to the first pictio if 	*/
					     /* the number 48 is exceeded	 					*/
    imf = ImgLoadImage("WSE11-ECSF_"ic".jpg")	/* loading another prepared flag-picture (with black background) 	*/
    rc  = ImgSetTool(imf, "SELMAGIC")
    "PAUSE" 					/* second operater action (see above)					*/
    rc  = ImgCopyImage(imf)
    rc  = ImgRevertImage(imf,TRUE)		/* copy to clipboard							*/
    rc  = ImgCloseImage(imf)
    rc  = ImgPasteImage(ima,0,599,466,TRUE)	/* paste the magic selection (flag without background) to the second 	*/
						/* flagpole 								*/
  
    ic=(i+23)//48+1				/* calculating the number of the third flag picture with a distance 	*/
						/* of 24 pictures to the first 						*/
    ic=5000+ic
    imf = ImgLoadImage("WSE11-ECSF_"ic".jpg")	/* loading another prepared flag-picture (with black background) 	*/
    rc  = ImgSetTool(imf, "SELMAGIC")
    "PAUSE" 					/* third operater action (see above)					*/
    rc  = ImgCopyImage(imf)
    rc  = ImgRevertImage(imf,TRUE)		/* copy to clipboard							*/
    rc  = ImgCloseImage(imf)
    rc  = ImgPasteImage(ima,0,1025,474,TRUE)	/* paste the magic selection (flag without background) to the 3rd flagpole*/
  
    ic=(i+35)//48+1				/* calculating the number of the third flag picture with a distance 	*/
						/* of 36 pictures to the first 						*/
    ic=5000+ic
    imf = ImgLoadImage("WSE11-ECSF_"ic".jpg")	/* loading another prepared flag-picture (with black background) 	*/
    rc  = ImgSetTool(imf, "SELMAGIC")
    "PAUSE" 					/* fourth operater action (see above)					*/
    rc  = ImgCopyImage(imf)
    rc  = ImgRevertImage(imf,TRUE)		/* copy to clipboard							*/
    rc  = ImgCloseImage(imf)
    rc  = ImgPasteImage(ima,0,1344,475,TRUE)	/* paste the magic selection (flag without background) to the 3rd flagpole*/
    rc  = ImgSaveImage(ima,adat,"BMP","NONE")	/* save one picture of the Assumburg with 4 eCS flags to be used in 	*/
						/* the 2nd cycle 							*/
    rc  = ImgCloseImage(ima)			/* close castle-picture to free memory					*/
  end

end

/* -------------------------------------------------------------------- */
/*									*/
/* Second program cycle							*/
/*									*/
/* It creates all pictures for creating the video (size 640x480)	*/
/* Depending on the cycle's TO-value the output pictures contain seve-	*/
/* ral sequences of waving flags (provides by modulo operation for	*/
/* selecting the input files. The creation of those sequences is over-	*/
/* layed by other actions like						*/
/*	- zoom in (selecting the center area and cropping the rest)	*/
/*	- open the door							*/
/*	- more actions may be added					*/
/*									*/
/* -------------------------------------------------------------------- */

j=0					/* distance of selected area in the center from left border (used for zooming)	*/
k=0					/* distance of selected area in the center from lower border (used for zooming)	*/
/*j=480*/
/*k=360*/

do i=0 to 1199				/* number of output pictures (825 pictures = 33 sec play time)			*/
  say " "
  call charout ,right(i,4)"  "		/* info for operator								*/
  imod=i//48				/* wrap to 1st input picture (repetition of waving-flag-sequence)		*/
  icm=5001+imod;      edat="AB-"icm".bmp"  /* name of input picture (Assumburg with 4 flags 1600x1200)			*/
  call charout ,"AB-"icm"  "
  icm=right(i,4,"0"); adat="AB-"icm".jpg"  /* name of output picture: (possible modified) Assumburg with 4 flags 640x480*/
  ima = ImgLoadImage(edat,TRUE)		   /* Loading input picture (Assumburg with 4 flags 1600x1200)			*/

  /* ..................................................................	*/
  /* Routine for openening the door					*/
  /* 									*/
  /* Both door leafs open in an angel from 0 to 90 un 96 steps	*/
  /* The visible part of a door leaf is calculated by cos-function. The */
  /* resulting values of door leaf width vary from 42 pixels (0) to 1	*/
  /* pixel (90)							*/
  /* The partionally opened door leafes allow a view into the interior.	*/
  /* After having reached the angel of 90 this routine only provides 	*/
  /* displaying the interior (the kastel's reception area)		*/
  /* ..................................................................	*/

  if i>383 & i<1150 then 				/* starting with picture 384 					*/
  do
    imo = ImgLoadImage("Tuer-offen-4k.jpg",TRUE)	/* Open interior picture (view when the door is open)		*/
    if imo=0 then say "Tuer-offen-4k.jpg fehlt"
    rc  = ImgCopyImage(imo)				/* copy to clipboard						*/
    rc  = ImgCloseImage(imo)				/* close interior-picture to free memory			*/
    rc  = ImgPasteImage(ima,0,794,389,TRUE)		/* paste to the area where the door is located			*/
    if i<483 | i>1059 then 
    do
      breitel = format(cos(pi()/2*(i-384)/96)*42,,0)	/* calculating the width of the visible part of left door leave */
      if breitel<1 then breitel=1			/* door is open, no further action				*/
      breiter = format(cos(pi()/2*(i-389)/96)*42,,0)	/* calculating the width of the visible part of right door	*/
      if breiter<1 then breiter=1			/* door is open, no further action				*/
      posr   = 42-breiter				/* position of left border of the right door leaf		*/
      breitelc=right(breitel,2,"0")			/* width with leading zero					*/
      breiterc=right(breiter,2,"0")			/* width with leading zero					*/
      imt = ImgLoadImage("Tuer-zu-l-"breitelc".jpg",TRUE) /* loading picture of left door leaf with correct width	*/
      if imt=0 then say "Tuer-zu-l-"breitelc".jpg fehlt"
      rc  = ImgCopyImage(imt)				/* copy to clipboard						*/
      rc  = ImgCloseImage(imt)				/* close left-door-leaf-picture to free memory			*/
      rc  = ImgPasteImage(ima,0,794,389,TRUE)		/* paste to the left part of thearea where the door is located	*/
      imt = ImgLoadImage("Tuer-zu-r-"breiterc".jpg",TRUE) /* loading picture of right door leaf with correct width	*/
      if imt=0 then say "Tuer-zu-r-"breiterc".jpg fehlt"
      rc  = ImgCopyImage(imt)				/* copy to clipboard						*/
      rc  = ImgCloseImage(imt)				/* close right-door-leaf-picture to free memory			*/
      rc  = ImgPasteImage(ima,0,794+42+posr,389,TRUE)	/* paste to the right part of the area where the door is located*/
      call charout ,"door: l="breitel", r="breiter"  "
    end
  end

  /* ..................................................................	*/
  /* Routine for displaying WSE-Logo					*/
  /* ..................................................................	*/

  if i>800  then
  do
    br=i-800+2
    if br>250 then br=250
    /*ho=format(br*0.444,,0)*/
    ho=format(br*0.5,,0)
    /*ho=br-139; if ho<1 then ho=1*/ 
    iml = ImgLoadImage("WarpstockEurope2013-50.bmp",TRUE)	/* Open WSE-Logo		*/
    if iml=0 then say "WarpstockEurope2013-50.bmp fehlt" 
    rc  = ImgResizeImage(iml,1,br,ho,TRUE)
    rc  = ImgCopyImage(iml)
    rc  = ImgRevertImage(iml,TRUE)				/* copy to clipboard		*/
    rc  = ImgCloseImage(iml)					/* close WSE-Logo to free memory*/
    x1  = format(125-br/2,,0)
    y1  = format(ho*1.5,,0)
    /*rc  = ImgPasteImage(ima,0,730+x1,515+y1,TRUE)*/		/* paste above the door		*/
    rc  = ImgPasteImage(ima,0,730+x1,494+y1,TRUE)		/* paste above the door		*/
    call charout ,"Logo w:"br", y:"515+y1"  "
  end


  /* ..................................................................	*/
  /* Routine for zooming in						*/
  /* 									*/
  /* Starting with picture 201 the next 480 pictures are cropped on all */
  /* four edges. j represents the width of the cropped space on the left*/
  /* and right side, k represents the space above and below.		*/
  /* After 480 pictures all resulting pictures have a size of 640x480	*/
  /* ..................................................................	*/

  if i>199 then						/* starting with picture 201 					*/
  do
    if j<480 then					/* decreasing the size for the next 480 pictures		*/
    do
      j=j+1						/* width decreases by 1						*/
      k=format(j/4*3,,0)				/* height decreases by 1 on 3 of 4 frames			*/
    end
    rc = ImgSelectRectangle(ima,2,j,k,1600-j-j,1200-k-k)	/* selecting the center of the picture			*/
    Rc = ImgCropImage(ima)				/* cropping the unselected area at the vor edges		*/
    call charout ,"zoom: "1600-j-j"x"1200-k-k"  "	/* info for operator						*/
  end


  /* ..................................................................	*/
  /* Routine for fading out						*/
  /* ..................................................................	*/

  if i>1149 then
  do
    rc = ImgSetImageBC(ima,(1150-i)*5,0)
    fadeVal=(1150-i)*5
    call charout ,"fade out: "fadeval"  "
  end

  /* ..................................................................	*/
  /* Routine for resizing the picture to 640x480			*/
  /* 									*/
  /* All pictures are finally resized to 640x480			*/
  /* Together with the previous cropping this results in the zoom-	*/
  /* effect.								*/
  /* ..................................................................	*/

  rc =  ImgResizeImage(ima,1,640,480,TRUE)
  rc =  ImgSaveImage(ima,adat,"JPG","JPEG")
  rc =  ImgCloseImage(ima)
end

/* -------------------------------------------------------------------- */
/* Routine end								*/
/* -------------------------------------------------------------------- */
"copy Abspann.bmp Abspann-tmp.bmp 1>nul 2>nul "

do i=1200 to 1999 /* 1200: Abspann, 1250: Schrift */
  say " "
  call charout ,right(i,4)"  "			/* info for operator				*/
  edat="Abspann-tmp.bmp"			/* (1600x1200)					*/
  icm=right(i,4,"0"); adat="AB-"icm".jpg"	/* name of output picture: 640x480		*/
  call charout ,"end cred.  "
  ima = ImgLoadImage(edat,TRUE)
  if ima=0 then 
  do 
    say edat" fehlt"
    exit 
  end

  /* ..................................................................	*/
  /* Routine for fading in						*/
  /* ..................................................................	*/

  if i<1250 then
  do
    rc = ImgSetImageBC(ima,(i-1250)*5,0) /* Fade in */
    fadeVal=(i-1250)*5
    call charout ,"fade in: "fadeval"  "

  end

  /* ..................................................................	*/
  /* Routine for writing text						*/
  /* ..................................................................	*/

  /* Values ll1...ll4 Abspann-txt4.bmp: 670, 760, 780, 760 */
  pl = 15							/* progress writing textlines (in pixels) 	*/
  l1 = 1250; ll1 = 780; l1p = format(l1+ll1/pl,,0)		/* Begin, length, number frames 1st textline	*/
  l2 = l1p;  ll2 = 780; l2p = format(l2+ll2/pl,,0)		/* Begin, length, number frames 2nd textline	*/
  l3 = l2p;  ll3 = 750; l3p = format(l3+ll3/pl,,0)		/* Begin, length, number frames 3rd textline	*/
  l4 = l3p;  ll4 = 560; l4p = format(l4+ll4/pl,,0)		/* Begin, length, number frames 4th textline	*/
  

  if i>=l1 & i<=l1p then					/* output first text line 	*/
  do
    imt = ImgLoadImage("Abspann-txt4.bmp",TRUE)
    if imt=0 then say "Abspann-txt4.bmp fehlt"
    rc  = ImgSelectRectangle(imt,2,0,2850,84+(i-l1)*pl,150)	/* selecting part of the text	*/
    rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
    rc =  ImgCloseImage(imt)
    rc  = ImgPasteImage(ima,0,480,585,TRUE)			/* paste 			*/
    call charout ,"textline 1 w: "84+(i-l1)*pl"  "
  end

  if i=l2 then rc =  ImgSaveImage(ima,"Abspann-tmp.bmp","BMP","NONE")	/* save pict with 1st line */

  if i>=l2 & i<=l2p then					/* output second text line 	*/
  do
    imt = ImgLoadImage("Abspann-txt4.bmp",TRUE)
    rc  = ImgSelectRectangle(imt,2,0,2714,84+(i-l2)*pl,150)	/* selecting part of the text	*/
    rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
    rc =  ImgCloseImage(imt)
    rc  = ImgPasteImage(ima,0,480,449,TRUE)			/* paste 			*/
    call charout ,"textline 2 w: "84+(i-l2)*pl"  "
  end

  if i=l3 then rc =  ImgSaveImage(ima,"Abspann-tmp.bmp","BMP","NONE")	/* save pict with 2nd line */

  if i>=l3 & i<=l3p then					/* output third text line 	*/
  do
    imt = ImgLoadImage("Abspann-txt4.bmp",TRUE)
    rc  = ImgSelectRectangle(imt,2,0,2585,84+(i-l3)*pl,150)	/* selecting part of the text	*/
    rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
    rc =  ImgCloseImage(imt)
    rc  = ImgPasteImage(ima,0,480,320,TRUE)			/* paste 			*/
    call charout ,"textline 3 w: "84+(i-l3)*pl"  "
  end

  if i=l4 then rc =  ImgSaveImage(ima,"Abspann-tmp.bmp","BMP","NONE")	/* save pict with 3rd line */

  if i>=l4 & i<=l4p then					/* output fourth text line 	*/
  do
    imt = ImgLoadImage("Abspann-txt4.bmp",TRUE)
    rc  = ImgSelectRectangle(imt,2,0,2441,84+(i-l4)*pl,150)	/* selecting part of the text	*/
    rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
    rc =  ImgCloseImage(imt)
    rc  = ImgPasteImage(ima,0,480,176,TRUE)			/* paste 			*/
    call charout ,"textline 4 w: "84+(i-l4)*pl"  "
  end

  if i=1550 then "copy Abspann.bmp Abspann-tmp.bmp 1>nul 2>nul "

  /* ..................................................................	*/
  /* Routine for srolling up						*/
  /* ..................................................................	*/

  if i>l4p & i<1936 then					/* 	*/
  do
    imt = ImgLoadImage("Abspann-txt4.bmp",TRUE)
    rc  = ImgSelectRectangle(imt,2,0,2275+(l4p-i)*4,999,825)	/* 725 */   /* selecting 	*/
    rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
    rc =  ImgCloseImage(imt)
    rc  = ImgPasteImage(ima,0,480,10,TRUE)			/* paste 			*/
    scrollVal=(l4p-i)*4
    call charout ,"scroll up "scrollVal"  "
  end

  /* ..................................................................	*/
  /* Routine for fading out						*/
  /* ..................................................................	*/

  if i>1934 & i<2000 then					/* 	*/
  do
    imt = ImgLoadImage("Abspann-txt4.bmp",TRUE)
    rc  = ImgSelectRectangle(imt,2,0,2275+(l4p-1935)*4,999,825)	/* 725 */   /* selecting 	*/
    rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
    rc  = ImgPasteImage(ima,0,480,10,TRUE)			/* paste 			*/
    scrollVal=(l4p-1935)*4
    call charout ,"display  "scrollVal"  "
    if i>1949 then 
    do
      rc  = ImgSelectRectangle(imt,2,532,65,319,84)	  	/* selecting 	copyleft info	*/
      rc  = ImgCopyImage(imt)					/* copy to clipboard		*/
      rc  = ImgPasteImage(ima,0,880,25,TRUE)			/* paste 			*/
      call charout ,"copyleft  "
    end
    rc =  ImgCloseImage(imt)
    if i>1974 then  rc = ImgSetImageBC(ima,(1974-i)*10,0) /* Fade out */
    fadeVal=(1974-i)*10
    call charout ,"fade out: "fadeVal"  "
  end


  /* ..................................................................	*/
  /* Routine for resizing the picture to 640x480			*/
  /* 									*/
  /* All pictures are finally resized to 640x480			*/
  /* Together with the previous cropping this results in the zoom-	*/
  /* effect.								*/
  /* ..................................................................	*/

  rc =  ImgResizeImage(ima,1,640,480,TRUE)
  rc =  ImgSaveImage(ima,adat,"JPG","JPEG")
  rc =  ImgCloseImage(ima)
end

/* -------------------------------------------------------------------- */
/*									*/
/* cleaning up								*/
/*									*/
/* -------------------------------------------------------------------- */

rc = RXFUNCDROP("IMPREXX")
rc = RxFuncdrop('REXXMATH')

/* -------------------------------------------------------------------- */
/*									*/
/* creating the video							*/
/*									*/
/* -------------------------------------------------------------------- */

/* putting the jpegs together to an mpeg-Video				*/

say
say "[1;33;40mCreate video from images AB-????.jpg --> AB-flag2.mpeg[0m"
say

"\osutil\ffmpeg-r18731\ffmpeg.exe -y -f image2 -i AB-%%04d.jpg -b 22000k AB-flag2.mpeg"	

/* adding sound and saving as mp4					*/

say
say '[1;33;40mAdd sound "gmsa-pypkonspyp.mp3" to video AB-flag2.mpeg --> AB-flag2.mp4[0m'
say
"\osutil\ffmpeg-r18731\ffmpeg.exe -y -i AB-flag2.mpeg -i gmsa-pypkonspyp.mp3 -ab 128k -b 22000k -vol 100 AB-flag2.mp4"

exit

